<?php

namespace Amazingcat\Labels\Controller\Adminhtml\Actions;

use Amazingcat\Labels\Model\ResourceModel\DefaultLabel\CollectionFactory;
use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\Result\Redirect;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Filesystem;
use Magento\Framework\Filesystem\Driver\File;

/**
 * Class Delete
 * @package Amazingcat\Labels\Controller\Adminhtml\Actions
 */
class Delete extends Action
{
    /**
     * @var CollectionFactory
     */
    protected $labelCollectionFactory;

    /**
     * @var File
     */
    protected $file;

    /**
     * @var Filesystem
     */
    protected $fileSystem;

    /**
     * Delete constructor.
     * @param Context $context
     * @param CollectionFactory $labelCollectionFactory
     * @param File $file
     * @param Filesystem $fileSystem
     */
    public function __construct(
        Context $context,
        CollectionFactory $labelCollectionFactory,
        File $file,
        Filesystem $fileSystem
    ) {
        $this->labelCollectionFactory = $labelCollectionFactory;
        $this->file = $file;
        $this->fileSystem = $fileSystem;

        parent::__construct($context);
    }

    /**
     * @return ResponseInterface|Redirect|ResultInterface
     */
    public function execute()
    {
        $excluded = $this->getRequest()->getParam('excluded', []);
        $selected = $this->getRequest()->getParam('selected', []);
        $itemsToDelete = [];

        $collection = $this->labelCollectionFactory->create();

        if ($excluded === false) {
            $itemsToDelete = $collection->getItems();
        } else {
            if (!empty($excluded)) {
                $itemsToDelete = $collection
                    ->addFieldToFilter('id', ['nin' => $excluded])
                    ->getItems();
            } else {
                if (!empty($selected)) {
                    $itemsToDelete = $collection
                        ->addFieldToFilter('id', ['in' => $selected])
                        ->getItems();
                } else {
                    $id = $this->getRequest()->getParam('id');
                    $itemsToDelete[] = $collection
                        ->addFieldToFilter('id', $id)
                        ->getFirstItem();
                }
            }
        }

        $redirect = $this->resultRedirectFactory->create();
        $url = $this->getUrl("default_labels/pages/grid");

        if (!empty($itemsToDelete)) {
            try {
                $this->_eventManager->dispatch('amazingcat_labels_defaultlabel_remove', ['items_to_delete' => $itemsToDelete]);

                foreach ($itemsToDelete as $itemToDelete) {
                    $itemToDelete->delete();
                }

                $this->messageManager->addSuccessMessage(
                    __('Selected items was successfully deleted')
                );
            } catch (\Exception $ex) {
                $this->messageManager->addErrorMessage($ex->getMessage());
            } finally {
                return $redirect->setUrl($url);
            }
        }

        $this->messageManager->addErrorMessage(__("Selected items doesn't exist"));

        return $redirect->setUrl($url);
    }
}
