<?php

namespace Amazingcat\Labels\Controller\Adminhtml\Actions;

use Amazingcat\Labels\Api\Data\DefaultLabelInterfaceFactory;
use Amazingcat\Labels\Model\DefaultLabel;
use Amazingcat\Labels\Model\ResourceModel\DefaultLabel as ResourceDefaultLabel;
use Amazingcat\Labels\Model\ResourceModel\DefaultLabel\CollectionFactory;
use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\Result\Redirect;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Exception\AlreadyExistsException;
use Magento\Framework\Filesystem;
use Magento\Framework\Filesystem\Directory\WriteInterface;

/**
 * Class Save
 * @package Amazingcat\Labels\Controller\Adminhtml\Actions
 */
class Save extends Action
{
    /**
     * @var DefaultLabelFactory
     */
    protected $labelFactory;

    /**
     * @var ResourceDefaultLabel
     */
    protected $labelResource;

    /**
     * @var CollectionFactory
     */
    protected $labelCollectionFactory;

    /**
     * @var WriteInterface
     */
    protected $mediaDirectory;

    /**
     * Save constructor.
     * @param Context $context
     * @param DefaultLabelInterfaceFactory $labelFactory
     * @param ResourceDefaultLabel $labelResource
     * @param CollectionFactory $labelCollectionFactory
     * @param Filesystem $filesystem
     * @throws \Magento\Framework\Exception\FileSystemException
     */
    public function __construct(
        Context $context,
        DefaultLabelInterfaceFactory $labelFactory,
        ResourceDefaultLabel $labelResource,
        CollectionFactory $labelCollectionFactory,
        FileSystem $filesystem
    ) {
        $this->labelFactory = $labelFactory;
        $this->labelResource = $labelResource;
        $this->labelCollectionFactory = $labelCollectionFactory;
        $this->mediaDirectory = $filesystem->getDirectoryWrite(DirectoryList::MEDIA);

        parent::__construct($context);
    }

    /**
     * @return ResponseInterface|Redirect|ResultInterface
     * @throws AlreadyExistsException
     */
    public function execute()
    {
        $data = $this->getRequest()->getPostValue();
        $labelEntity = !empty($data['id']) ? $this->getLabel($data['id']) : $this->labelFactory->create();
        $imageEntity = $data['logo_path'][0];
        $imagePath = '/media/' . $this->mediaDirectory->getRelativePath('catalog/product/defaultlabels/' . $imageEntity ['name']);

        $labelEntity
            ->setName($data['name'])
            ->setImagePath(str_replace(' ', '_', $imagePath))
            ->setPosition($data['position']);

        $this->_eventManager->dispatch('amazingcat_labels_defaultlabel_beforesave', ['label_entity' => $labelEntity]);

        $this->labelResource->save($labelEntity);

        $this->_eventManager->dispatch('amazingcat_labels_defaultlabel_aftersave', ['label_entity' => $labelEntity]);

        $this->messageManager->addSuccessMessage('Label created successfully');
        $redirect = $this->resultRedirectFactory->create();
        $url = $this->getUrl("default_labels/pages/grid");

        return $redirect->setUrl($url);
    }

    /**
     * @param int $id
     * @return DefaultLabel
     */
    protected function getLabel(int $id)
    {
        $model = $this->labelFactory->create();
        $this->labelResource->load($model, $id);

        return $model;
    }
}
