<?php

namespace Amazingcat\Labels\Controller\Adminhtml\Actions;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Filesystem;
use Magento\Framework\Message\ManagerInterface;
use Magento\MediaStorage\Model\File\UploaderFactory;

/**
 * Class Upload
 * @package Amazingcat\Labels\Controller\Adminhtml\Actions
 */
class Upload extends Action
{
    /**
     * @var JsonFactory
     */
    protected $jsonFactory;

    /**
     * @var Filesystem\Directory\WriteInterface
     */
    protected $mediaDirectory;

    /**
     * @var UploaderFactory
     */
    protected $uploaderFactory;

    /**
     * Upload constructor.
     * @param Context $context
     * @param JsonFactory $jsonFactory
     * @param Filesystem $filesystem
     * @param UploaderFactory $uploaderFactory
     * @param ManagerInterface $messageManager
     * @throws \Magento\Framework\Exception\FileSystemException
     */
    public function __construct(
        Context $context,
        JsonFactory $jsonFactory,
        FileSystem $filesystem,
        UploaderFactory $uploaderFactory,
        ManagerInterface $messageManager
    ) {
        $this->mediaDirectory = $filesystem->getDirectoryWrite(DirectoryList::MEDIA);
        $this->uploaderFactory = $uploaderFactory;
        $this->jsonFactory = $jsonFactory;
        $this->messageManager = $messageManager;

        parent::__construct($context);
    }

    /**
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $resultJson = $this->jsonFactory->create();

        try {
            $uploader = $this->uploaderFactory
                ->create(['fileId' => 'logo_path'])
                ->setAllowCreateFolders(true);

            $uploader->setAllowedExtensions(['jpg', 'ico', 'png', 'gif']);
            $uploader->setAllowRenameFiles(false);
            $path = $this->mediaDirectory->getAbsolutePath('/defaultlabels');
            $result = $uploader->save($path);
            $result['url'] = '/pub/media/defaultlabels/' . $result['file'];

            if ($result['file']) {
                $resultJson->setData(array_merge(['success' => true], $result));
            }
        } catch (\Exception $e) {
            $resultJson->setData([
                'success' => false,
                'message' => $e->getMessage() . $e->getFile() . " " . $e->getLine(),
            ])->setHttpResponseCode(400);
        } finally {
            return $resultJson;
        }
    }
}
