<?php

namespace Amazingcat\Labels\Setup;

use Magento\Catalog\Model\Category\Attribute\Backend\Image;
use Magento\Catalog\Model\ResourceModel\Eav\Attribute;
use Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface;
use Magento\Eav\Setup\EavSetupFactory;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\UpgradeDataInterface;

/**
 * Class UpgradeData
 * @package Amazingcat\Labels\Setup
 */
class UpgradeData implements UpgradeDataInterface
{
    /**
     * @var EavSetupFactory
     */
    protected $eavSetupFactory;

    /**
     * UpgradeData constructor.
     * @param EavSetupFactory $eavSetupFactory
     */
    public function __construct(
        EavSetupFactory $eavSetupFactory
    ) {
        $this->eavSetupFactory = $eavSetupFactory;
    }

    /**
     * @param ModuleDataSetupInterface $setup
     * @param ModuleContextInterface $context
     */
    public function upgrade(ModuleDataSetupInterface $setup, ModuleContextInterface $context)
    {
        $setup->startSetup();
        $eavSetup = $this->eavSetupFactory->create(['setup' => $setup]);

        if (version_compare($context->getVersion(), '1.1.6') < 0) {
            $eavSetup
                ->addAttribute(
                    \Magento\Catalog\Model\Product::ENTITY,
                    'default_label',
                    [
                        'type' => 'varchar',
                        'label' => 'Default Label',
                        'backend' => 'Magento\Eav\Model\Entity\Attribute\Backend\ArrayBackend',
                        'input' => 'select',
                        'source' => 'Amazingcat\Labels\Model\Source\DefaultLabelSource',
                        'required' => false,
                        'global' => Attribute::SCOPE_STORE,
                        'order' => 120,
                        'searchable' => false,
                        'filterable' => false,
                    ]
                );
        }

        if (version_compare($context->getVersion(), '1.1.11') < 0) {
            $eavSetup->addAttribute(
                \Magento\Catalog\Model\Product::ENTITY,
                'product_label',
                [
                    'group' => 'Product Details',
                    'type' => 'varchar',
                    'backend' => Image::class,
                    'label' => 'Product Label',
                    'input' => 'media_image',
                    'class' => '',
                    'global' => ScopedAttributeInterface::SCOPE_GLOBAL,
                    'visible' => true,
                    'required' => false,
                    'user_defined' => true,
                    'default' => '',
                    'searchable' => false,
                    'filterable' => false,
                    'comparable' => false,
                    'visible_on_front' => false,
                    'used_in_product_listing' => true,
                    'unique' => false,
                ]
            );
        }

        $setup->endSetup();
    }
}
